//Carlos Gershenson
//carlos@jlagunez.iquimica.unam.mx

import java.awt.*;
import java.io.*; 
import java.util.*;
import vrml.external.field.*;
import vrml.external.Node;
import vrml.external.Browser;
import vrml.external.exception.*;

public class Proteinat extends Frame implements Serializable {
  String id;//proteinat's name
  int type = 0;//0=generic; 1=extsignal; 2=receptor; 3=Gprot; 4=prot to be linked to Gprot
  int state = 0;//0=generic; 1=activated;
  int simpos =-1;//position in proteinats vector in Celulat
  boolean linked = false;
  float rp = 1;//raidus of perception
  float decayfactor = (float)0.003;
  Browser browser;
  Random rnd = new Random();
  float[] pos = new float[3]; //proteinat position
  float[] rot = new float[4]; //proteinat orientation 
  float[] rot2 = new float[4];
  Node prot,prot2;
  EventInSFVec3f posEv; 
  EventInSFRotation rotEv; 
  EventInSFRotation rotEv2;
  boolean verbose = false;
  boolean immortality = false;
  Node material;
  EventInSFColor diffuseColor;
  final float[] activecolor = { (float)0.95, (float)0.05, (float)0.1 } ;
  float[] owncolor = new float[3];
  float[] color = new float[3];
  float[] color2 = new float[3];
  float[] pos2 = new float[3];
  float Nmax,Smax,Emax,Wmax;//maximum movement coords (bounds)
  TextArea output = null;
  TextField decayf;
  
  public Proteinat(String name, int spos, int ntype, int nstate, 
  float N, float S, float E, float W, float nrp,
  float[] npos, float [] ncolor, long t) {
    super ("Proteinat "+name);
    id = name;
    simpos = spos;
    type = ntype;
    state = nstate;
    rp = nrp;
    Nmax = N;
    Smax = S;
    Emax = E;
    Wmax = W;
    pos[0] = npos[0];
    pos[1] = npos[1];
    pos[2] = npos[2];
    owncolor[0] = color[0] = ncolor[0];
    owncolor[1] = color[1] = ncolor[1];
    owncolor[2] = color[2] = ncolor[2];
    setFont(new Font("TimesRoman",Font.PLAIN,12));
    setBackground(Color.white);
    setLayout(new FlowLayout());
    add(new Button("Clear Output")); 
    add(new Button("Verbose"));
    add(new Label("  "));
    add(new Button("Hide Window"));
    output = new TextArea(6, 40);
    output.setEditable(false);
    output.setFont(new Font("TimesRoman",Font.PLAIN,12));
    add(output);
    output.appendText("Proteinat " + name + " created at t = "+String.valueOf(t)+"\n");
    add(new Button("Activation"));
    add(new Label("Decay:"));
    decayf = new TextField(String.valueOf(decayfactor),4); 
    add(decayf);
    add(new Button("Set"));
    resize(290,250);
  }
  
  public void ini() {
    browser = (Browser) Celulat.browser;
    posEv.setValue(pos);
  } 
  
  public boolean action(Event evt, Object whichAction) {
    boolean ok = true; 
    if (evt.target instanceof Button) {
      Button b = (Button) evt.target;
      if (b.getLabel() == "Verbose") {
        if(verbose)
          output.appendText("OK, I'll be quiet.\n");
        else
          output.appendText("To inform and serve, sir!\n");
        verbose =! verbose;
      }
      else if (b.getLabel() == "Hide Window") {
        this.hide();
      }
      else if (b.getLabel() == "Activation") {
        if(state == 0) {
          activate();
        } else {
          deactivate();
        }
      }
      else if (b.getLabel() == "Set") {
        float prov =-1;
        try {
          prov = Float.valueOf(decayf.getText()).floatValue();
        }
        catch (NumberFormatException e) {
          output.appendText("Please fill the fileds correctly!\n");
          ok = false;
        }
        if (ok) {
          if ((prov < 0)||(prov > 1)) {
            output.appendText("Decay factor must be between 0 and 1.\n");
          } else {
            decayfactor = prov;
            output.appendText("New decay factor: "+String.valueOf(decayfactor)+"\n");
          }
        }
      }
      else if (b.getLabel() == "Clear Output") {
        output.setText("\nNice and clean...\n");
      }
    }
    return true;
  }
  
  boolean near(Proteinat p) { //checks if p is nearer than rp
    if(Math.sqrt(Math.pow(pos[0]-p.pos[0],2)+Math.pow(pos[2]-p.pos[2],2)) <= rp)
      return true;
    else
      return false;
  }
  void brownian() { //Brownian movement of proteins within their bounds
    float z,x;
    x = pos[0]-1+(2*rnd.nextFloat());
    z = pos[2]-1+(2*rnd.nextFloat());
    if((z > Nmax)&&(z < Smax)) pos[2] = z;
    if((x > Wmax)&&(x < Emax)) pos[0] = x;
  }
  boolean trylink(int prottype) { //proteinat trying to link prottype
    for(int i = 0;i < Celulat.maxproteinats;i++) {
      if(Celulat.proteinats[i].type == prottype)
        if ((near(Celulat.proteinats[i]))&&(Celulat.proteinats[i].state == 1)) {
        Celulat.proteinats[i].linked = true;
        if (Celulat.proteinats[i].verbose)
          Celulat.proteinats[i].output.appendText("Proteinat linked\n");
        return true;
      }
    }
    return false;
  }
  
  void activate() {
    state = 1;
    if (verbose)output.appendText("Proteinat activated\n");
    diffuseColor.setValue(activecolor);
  }
  
  void deactivate() {
    state = 0;
    if (verbose)output.appendText("Proteinat deactivated\n");
    if(linked) {
      linked = false;
      if (verbose)output.appendText("Proteinat unlinked\n");
    }
    diffuseColor.setValue(owncolor);
  }
  
  void go() {
    switch(type) {
      case 0:brownian();
      if ((decayfactor > rnd.nextFloat())&&(state == 1)) { //decay activity
        deactivate();
      }
      break;
      case 1:if( ! linked)
        brownian();
      if ((decayfactor > rnd.nextFloat())&&(state == 1)) { //decay activity
        deactivate();
      }
      break;
      case 2:if(state == 0) {
        if (trylink(1)) 
          activate();
      }
      if ((decayfactor > rnd.nextFloat())&&(state == 1)) { //decay activity
        deactivate();
      }
      break;
      case 3:if(state == 0) {
        if (trylink(2)) 
          activate();
        if( ! linked)
          brownian();
      }
      if ((decayfactor > rnd.nextFloat())&&(state == 1)) { //decay activity
        deactivate();
      }
      break;
      case 4:if(state == 0) {
        if (trylink(3)) { 
          activate();
          Celulat.addcalcium();
        }
        if( ! linked)
          brownian();
      }
      if ((decayfactor > rnd.nextFloat())&&(state == 1)) { //decay activity
        Celulat.rmcalcium();
        deactivate();
      }
      break;
    }
    if (Celulat.refresh()) {
      update(true);
      if(verbose) {
        output.appendText("Proteinat at x:"+String.valueOf(pos[0])+
        ", z:"+String.valueOf(pos[2])+"\n");
      }
    }
  }
  
  public void update(boolean insim) {
    posEv.setValue(pos);
    //rotEv.setValue(rot);
    if ( ! insim) {
      posEv.setValue(pos);
      //rot2[0] = 1; rot2[1] = 0; rot2[2] = 0; rot2[3] = 0;
      if (state == 0) {
        diffuseColor.setValue(owncolor);
      } else {
        diffuseColor.setValue(activecolor);
      } 
    }
  }
  
}